#include <iostream>
#include <string>
#include <vector>

using namespace std;

const int CTE_LIKE=false;

/************************************************************
 * specialized functions and hard constraints
 * for sequence alignment
 */

/*
  Parameter/data for Scoring
*/
class ScoringInfo {
public:
    const string seqA;
    const string seqB;

    const int gamma;
    const int match;
    const int mismatch;
    
    int sigma(uint i, uint j) const {
	if (i==seqA.size()+1 && j==seqB.size()+1) return 0; //special case 
	return (seqA[i-1]==seqB[j-1])?match:mismatch;
    }

    ScoringInfo(const string _seqA, const string _seqB, int _gamma, int _match, int _mismatch)
	: seqA(_seqA), seqB(_seqB), gamma(_gamma), match(_match), mismatch(_mismatch) {}
};


int align(const string &seqA, const string &seqB, ScoringInfo *sc) {
    
    int n=seqA.size();
    int m=seqB.size();
    
    vector<vector<int> > M;
    
    M.resize(n+1);
    for (int i=0; i<=n; i++) M[i].resize(m+1);
    
    M[0][0]=0;
    for (int i=1; i<=n; i++) M[i][0] = i*sc->gamma;
    for (int j=1; j<=m; j++) M[0][j] = j*sc->gamma;
    
    if (CTE_LIKE) {
	/* this is incorrect !!! WHY ??? */
	for (int i=1; i<=n; i++) {
	    for (int j=1; j<=m; j++) {
		M[i][j] = INT_MIN;
		for (int j_pr=0; j_pr<j; j_pr++) {
		    M[i][j] =
			max (M[i][j],
			     max( M[i-1][j_pr] + ( j - j_pr - 1 ) * sc->gamma + sc->sigma(i,j),
				  M[i-1][j]+sc->gamma )
			    );
		}
	    }
	}
    } else {
	for (int i=1; i<=n; i++) {
	    for (int j=1; j<=m; j++) {
		M[i][j] =
		    max(M[i-1][j-1]+sc->sigma(i,j),
			max(M[i-1][j]+sc->gamma,
			    M[i][j-1]+sc->gamma));
	    }
	}
    }
    
    
    return M[n][m];
}


int
main(int argc, char **argv) {
    string seqA;
    string seqB;
    
    if (argc==3) {
	seqA=argv[1];
	seqB=argv[2];
    } else {
	cerr << "USAGE "<<argv[0]<<" <seqA> <seqB>"<<endl;
	exit(-1);
    }

    ScoringInfo myScoringInfo(seqA,seqB,-2,3,-1);


    int result = align(seqA,seqB,&myScoringInfo);


    cout << "Result: "<<result<<endl;
    
    
    exit(0);
}
