#ifndef VARIABLE_HH
#define VARIABLE_HH

#include "auxiliary.hh"

/* The domain class specifies a set of integers
   
   Operations:
   * traversal
   * upper bounding, lower bounding
   * intersection
   * copy

   Implemented by lower, upper bound
   
   CURRENTLY only complete ranges can be represented 
*/
class Variable;

class Domain {
protected:
    int lb_;
    int ub_;
public:
    Domain():lb_(0),ub_(-1) {}
    Domain(int lb, int ub):lb_(lb),ub_(ub) {}
    
    int first() {//get first domain value or -1 if there is no first domain value
	return lb_;
    }
    int next(int x) { //get next domain value or -1 if there is no next domain value
	return (x<ub_)?(x+1):-1;
    }
    int size() const { // size of domain
	return ub_-lb_+1;
    }

    void set_dom(const Domain &d) {lb_=d.lb_; ub_=d.ub_;}
    const Domain &get_dom() const {return *this;}

    int get_lbound() const {return lb_;} // set lower bound
    int get_ubound() const {return ub_;} // set upper bound

    bool set_lbound(int lb) {lb_=lb; return ub_>=lb_;} // set lower bound
    bool set_ubound(int ub) {ub_=ub; return ub_>=lb_;} // set upper bound

    bool set_geq(int lb) {lb_=std::max(lb_,lb); return ub_>=lb_;} // change lower bound
    bool set_leq(int ub) {ub_=std::min(ub_,ub); return ub_>=lb_;} // change upper bound
    
    bool restrict_dom(const Domain &d) {return set_geq(d.lb_) && set_leq(d.ub_);} // restrict own domain to given domain
    
    void set_singleton(int x) {// set to singleton domain {x}
	ub_=lb_=x;
    }
    int get_singleton() const { // set to singleton domain {x}
	//assert(lb_ == ub_);
	return(lb_); 
    }

    bool is_singleton() const {return lb_==ub_;}
  
    void print_debug() const;
};


/*
  A variable has a domain and a name
  (the name is only for output/debugging, not necessarily unique)
*/

class Variable : public Domain {
    const std::string name_;
public:
    Variable(): Domain(0,0),name_() {}
    Variable(const std::string &name, int lb, int ub):Domain(lb,ub),name_(name) {}


    void print_debug() const;    
    const std::string &get_name() const {return name_;}

};



#endif // VARIABLE_HH
